/*
 * 
 */
package casa.JION.lease;

import java.rmi.RemoteException;

import casa.JION.space.JavaSpace;
import casa.JION.space.JavaSpaceSingleton;
import casa.dodwan.docware.Descriptor;

import net.jini.core.lease.LeaseDeniedException;
import net.jini.core.lease.LeaseMap;
import net.jini.core.lease.UnknownLeaseException;

/**
 * The Lease interface defines a type of object that is returned to the
 * lease holder and issued by the lease grantor.  Particular instances of
 * the Lease type will be created by the grantors of a lease, and returned
 * to the holder of the lease as part of the return value from a call that
 * allocates a leased resource.  The call that requests a leased resource
 * will typically include a requested duration for the lease.  If the request
 * is for a particular duration, the lease grantor is required to grant a
 * lease of no more than the requested period of time. A lease may be granted
 * for a period of time shorter than that requested.
 *
 * @author Sun Microsystems, Inc.
 *
 * @since 1.0
 */
public class Lease implements net.jini.core.lease.Lease{
	
	/** The js. */
	private JavaSpace js =JavaSpaceSingleton.getInstance();
	
	/** The id of the entry controlled by this lease. */
	protected String entryID;
	/**
	 * The lease expiration.
	 */
	protected long serialForm;
	
	/**
	 * Serialization format for the expiration.
	 */
	protected int serialFormat=net.jini.core.lease.Lease.DURATION;
	
	/**
	 * Instantiates a new lease.
	 *
	 * @param entryID the entry id
	 * @param duration the duration
	 */
	public Lease(String entryID,long duration){
		this.entryID=entryID;
		if (duration>0)
			serialForm=duration;
		else
			if (duration==Lease.ANY)
				serialForm=Lease.FOREVER;
			else
				throw new IllegalArgumentException("Lease time can be only positive or equal to Lease.ANY");
	}

	/**
	 * Instantiates a new lease.
	 *
	 * @param desc the desc
	 */
	public Lease(Descriptor desc){
		this.entryID=desc.getAttribute("ENTRY_ID");
		this.serialForm=Long.valueOf(desc.getAttribute("SERIAL_FORM")).longValue();
		this.serialFormat=Integer.valueOf(desc.getAttribute("SERIAL_FORMAT")).intValue();
	}

    /**
     * Returns a boolean indicating whether or not the lease given as a
     * parameter can be batched (placed in the same LeaseMap) with the
     * current lease.  Whether or not two Lease objects can be batched
     * is an implementation detail determined by the objects.
     * 
     * @param lease the <tt>Lease</tt> to be evaluated
     * @return a boolean indicating whether or not the lease given as a
     *         parameter can be batched (placed in the same LeaseMap) with 
     *         the current lease
     */
	public boolean canBatch(net.jini.core.lease.Lease lease) {
		// TODO Auto-generated method stub
		return true;
	}


    /**
     * Creates a Map object that can contain leases whose renewal or
     * cancellation can be batched, and adds the current lease to that map.
     * The current lease is put in the map with the duration value given
     * by the parameter.
     *
     * @param duration the duration to put into a Long and use as the
     * value for the current lease in the created LeaseMap
     *
     * @return the created <tt>LeaseMap</tt> object
     */
	public LeaseMap createLeaseMap(long duration) {
		return new casa.JION.lease.LeaseMap(this, duration);
	}


	/** {@inheritDoc} */
	public long getExpiration() {
		return serialForm;
	}


    /**
     * Returns the format that will be used to serialize the lease.
     *
     * @return an <tt>int</tt> representing the serial format value
     * @see #setSerialFormat
     */
	public int getSerialFormat() {
		return serialFormat;
	}


    /**
     * Used by the lease holder to indicate that it is no longer interested
     * in the resource or information held by the lease.  If the leased
     * information or resource could cause a callback to the lease holder
     * (or some other object on behalf of the lease holder), the lease
     * grantor should not issue such a callback after the lease has been
     * cancelled.  The overall effect of a cancel call is the same as
     * lease expiration, but instead of happening at the end of a pre-agreed
     * duration it happens immediately.
     *
     * @throws UnknownLeaseException the lease being cancelled is unknown
     *         to the lease grantor
     * @throws RemoteException
     */
	public void cancel() throws UnknownLeaseException, RemoteException {
		js.doCancel(this);
		
	}
	

    /**
     * Used to renew a lease for an additional period of time, specified in
     * milliseconds.  This duration is not added to the original lease, but
     * is used to determine a new expiration time for the existing lease.
     * If the renewal is granted this is reflected in value returned by
     * getExpiration.  If the renewal fails, the lease is left intact for
     * the same duration that was in force prior to the call to renew.
     *
     * @param duration the requested duration in milliseconds
     *
     * @throws LeaseDeniedException the lease grantor is unable or
     *         unwilling to renew the lease
     * @throws UnknownLeaseException the lease being renewed is unknown
     *         to the lease grantor
     * @throws RemoteException
     */
	public void renew(long duration) throws LeaseDeniedException,
			UnknownLeaseException, RemoteException {
		if (duration <0)
			throw new IllegalArgumentException("Duration must be positive");
		serialForm = duration+System.currentTimeMillis();
		System.out.println(serialForm);
		js.doRenew(this);
	}
	



    /**
     * Sets the format to use when serializing the lease.
     *
     * @param format DURATION or ABSOLUTE
     * @see #getSerialFormat
     */
	public void setSerialFormat(int format){
			if (format != net.jini.core.lease.Lease.ABSOLUTE && format!= net.jini.core.lease.Lease.DURATION)
				throw new IllegalArgumentException("Format could be set to DURATION or ABSOLUTE only");
			serialFormat=format;
	}
	

	/** {@inheritDoc} */
	public String toString(){
		String format;
		if (serialFormat==1)
			format="DURATION";
		else
			format="ABSOLUTE";
		return "Serial_format: "+format+", Serial_form: "+serialForm+", The controlled entry:"+entryID;
	}
	
	/**
	 * Lease to descriptor.
	 *
	 * @return the descriptor
	 */
	public Descriptor leaseToDescriptor(){
		Descriptor desc = new Descriptor();
		desc.setAttribute("ENTRY_ID", this.entryID);
		desc.setAttribute("SERIAL_FORM", String.valueOf(this.serialForm));
		desc.setAttribute("SERIAL_FORMAT", String.valueOf(this.serialFormat));
		return desc;
	}
	
	/**
	 * Gets the entry id.
	 *
	 * @return the entry id
	 */
	public String getEntryID(){
		return entryID;
	}
	
	/**
	 * The main method.
	 *
	 * @param args the arguments
	 */
	public static void main(String[] args){
		try {
			Lease lease=new Lease("knkn",20000000);
			System.out.println(lease);
			lease.renew(40000000);
			System.out.println(lease);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

}
